define("epi-find/widget/DashboardRanges", [
        "dojo/date",
        "dojo/date/locale"
    ],
    function(
        // dojo
        date,
        locale
    ) {

    var Ranges = {};

    Ranges.last24hours = {
        name: "last24hours",
        interval: "hour",
        getStartDate: function(/*Date*/endDate) {
            // summary:
            //      Gets the start date based on end date
            // description:
            //      Minutes and seconds will be set to 0 to neutralise any DST effects
            // returns:
            //      Date
            var startDate = new Date(endDate.getTime());
            startDate.setDate(endDate.getDate() - 1);
            startDate.setUTCMinutes(0, 0);
            return startDate;
        },
        getIntervalEnd: function(/*Date*/intervalDate) {
            // summary:
            //      Gets the end of the current interval
            // description:
            //      This is most commonly used to retrieve a full interval span
            //      _including_ the specified date
            // returns:
            //      Date
            var endDate = date.add(intervalDate, Ranges.last24hours.interval, 1);
            endDate.setUTCMinutes(0, 0, 0);
            return endDate;
        },
        getRange: function(/*Object*/options) {
            // summary:
            //      Gets a range object based on passed options
            // description:
            //      The range object will include to, from and interval properties. If
            //      no object is passed, range will be calculated based on `to: new Date()`
            // returns:
            //      Object
            options = options || { to: new Date() };
            var to = Ranges.last24hours.getIntervalEnd(options.to);
            return {
                to: to,
                from: Ranges.last24hours.getStartDate(to),
                interval: Ranges.last24hours.interval
            };
        },
        labelFormat: function(/*Date*/date) {
            // summary:
            //      Returns a label formatted suitable for this range type
            // returns:
            //      string
            return locale.format(date, {
                selector: "date",
                datePattern: "HH" + (date.getHours() === 0 ? "\nd MMM" : "")
            });
        }
    };
    Object.freeze(Ranges.last24hours);

    Ranges.lastWeek = {
        name: "lastWeek",
        interval: "day",
        getStartDate: function(/*Date*/endDate) {
            // summary:
            //      Gets the start date based on end date
            // description:
            //      Minutes and seconds will be set to 0 to neutralise any DST effects
            // returns:
            //      Date
            var startDate = new Date(endDate.getTime());
            startDate.setDate(endDate.getDate() - 7);
            startDate.setUTCHours(0, 0, 0);
            return startDate;
        },
        getIntervalEnd: function(/*Date*/intervalDate) {
            // summary:
            //      Gets the end of the current interval
            // description:
            //      This is most commonly used to retrieve a full interval span
            //      _including_ the specified date
            // returns:
            //      Date
            var endDate = date.add(intervalDate, "day", 1);
            endDate.setUTCHours(0, 0, 0, 0);
            return endDate;
        },
        getRange: function(/*Object*/options) {
            // summary:
            //      Gets a range object based on passed options
            // description:
            //      The range object will include to, from and interval properties. If
            //      no object is passed, range will be calculated based on `to: new Date()`
            // returns:
            //      Object
            options = options || { to: new Date() };
            var to = Ranges.lastWeek.getIntervalEnd(options.to);
            return {
                to: to,
                from: Ranges.lastWeek.getStartDate(to),
                interval: Ranges.lastWeek.interval
            };
        },
        labelFormat: function(/*Date*/date) {
            // summary:
            //      Returns a label formatted suitable for this range type
            // returns:
            //      string
            return locale.format(date, {
                selector: "date",
                datePattern: "d" + (date.getDate() === 1 ? "\nMMMM" : "")
            });
        }
    };
    Object.freeze(Ranges.lastWeek);

    Ranges.lastMonth = {
        name: "lastMonth",
        interval: "day",
        getStartDate: function(/*Date*/endDate) {
            // summary:
            //      Gets the start date based on end date
            // description:
            //      Minutes and seconds will be set to 0 to neutralise any DST effects
            // returns:
            //      Date
            var startDate = new Date(endDate.getTime());
            startDate.setMonth(endDate.getMonth() - 1);
            startDate.setUTCHours(0, 0, 0);
            return startDate;
        },
        getIntervalEnd: function(/*Date*/intervalDate) {
            // summary:
            //      Gets the end of the current interval
            // description:
            //      This is most commonly used to retrieve a full interval span
            //      _including_ the specified date
            // returns:
            //      Date
            var endDate = date.add(intervalDate, "day", 1);
            endDate.setUTCHours(0, 0, 0, 0);
            return endDate;
        },
        getRange: function(/*Object*/options) {
            // summary:
            //      Gets a range object based on passed options
            // description:
            //      The range object will include to, from and interval properties. If
            //      no object is passed, range will be calculated based on `to: new Date()`
            // returns:
            //      Object
            options = options || { to: new Date() };
            var to = Ranges.lastMonth.getIntervalEnd(options.to);
            return {
                to: to,
                from: Ranges.lastMonth.getStartDate(to),
                interval: Ranges.lastMonth.interval
            };
        },
        labelFormat: function(/*Date*/date) {
            // summary:
            //      Returns a label formatted suitable for this range type
            // returns:
            //      string
            return locale.format(date, {
                selector: "date",
                datePattern: "d" + (date.getDate() === 1 ? "\nMMMM" : "")
            });
        }
    };
    Object.freeze(Ranges.lastMonth);

    Ranges.lastYear = {
        name: "lastYear",
        interval: "month",
        getStartDate: function(/*Date*/endDate) {
            // summary:
            //      Gets the start date based on end date
            // description:
            //      Minutes and seconds will be set to 0 to neutralise any DST effects
            // returns:
            //      Date
            var startDate = new Date(endDate.getTime());
            startDate.setYear(endDate.getFullYear() - 1);
            startDate.setUTCHours(0, 0, 0);
            return startDate;
        },
        getIntervalEnd: function(/*Date*/intervalDate) {
            // summary:
            //      Gets the end of the current interval
            // description:
            //      This is most commonly used to retrieve a full interval span
            //      _including_ the specified date
            // returns:
            //      Date
            var endDate = date.add(intervalDate, "day", 1);
            endDate.setUTCHours(0, 0, 0, 0);
            return endDate;
        },
        getRange: function(options) {
            // summary:
            //      Gets a range object based on passed options
            // description:
            //      The range object will include to, from and interval properties. If
            //      no object is passed, range will be calculated based on `to: new Date()`
            // returns:
            //      Object
            options = options || { to: new Date() };
            var to = Ranges.lastYear.getIntervalEnd(options.to);
            return {
                to: to,
                from: Ranges.lastYear.getStartDate(to),
                interval: Ranges.lastYear.interval
            };
        },
        labelFormat: function(/*Date*/date) {
            // summary:
            //      Returns a label formatted suitable for this range type
            // returns:
            //      string
            return locale.format(date, {
                selector: "date",
                datePattern: "MMM" + (date.getDate() === 1 && date.getMonth() === 0 ? "\nyyy" : "")
            });
        }
    };
    Object.freeze(Ranges.lastYear);

    /* TODO: Implement daterange span selection in Dashboard, post-beta */
    Ranges.dateRange = {
        name: "dateRange",
        interval: "day",
        startDate: null,
        endDate: null,
        getStartDate: function(/*Date*/from) {
            // summary:
            //      Gets the start date
            // description:
            //      Minutes and seconds will be set to 0 to neutralise any DST effects
            // returns:
            //      Date
            if (from) {
                var startDate = date.add(from, "day", 1);
                startDate.setUTCHours(0, 0, 0, 0);
                Ranges.dateRange.startDate = startDate;
            }
            if (!Ranges.dateRange.startDate) {
                var startDate = date.add(new Date(), "day", -1)
                startDate.setUTCHours(0, 0, 0, 0);
                Ranges.dateRange.startDate = startDate;
            }

            return Ranges.dateRange.startDate;
        },
        getIntervalEnd: function(/*Date*/intervalDate) {
            // summary:
            //      Gets the end of the current interval
            // description:
            //      This is most commonly used to retrieve a full interval span
            //      _including_ the specified date
            // returns:
            //      Date
            if (intervalDate) {
                var endDate = date.add(intervalDate, "day", 2);
                endDate.setUTCHours(0, 0, 0, 0);
                Ranges.dateRange.endDate = endDate;
            }
            if (!Ranges.dateRange.endDate) {
                var endDate = date.add(new Date(), "day", 1)
                endDate.setUTCHours(0, 0, 0, 0);
                Ranges.dateRange.endDate = endDate;
            }

            return Ranges.dateRange.endDate;
        },
        getRange: function(/*Object*/options) {
            // summary:
            //      Gets a range object based on passed options
            // description:
            //      The range object will include to, from and interval properties. If
            //      no object is passed, range will be calculated based on `to: new Date()`
            // returns:
            //      Object
            options = options || {};
            return {
                to: Ranges.dateRange.getIntervalEnd(options.to),
                from: Ranges.dateRange.getStartDate(options.from),
                interval: Ranges.dateRange.interval
            };
        },
        labelFormat: function(/*Date*/date) {
            // summary:
            //      Returns a label formatted suitable for this range type
            // returns:
            //      string
            return locale.format(date, {
                selector: "date",
                datePattern: "d" + (date.getDate() === 1 ? "\nMMMM" : "")
            });
        }
    };

    return Ranges;
});
