﻿define("epi-find/widget/WeightSelector", [
    "dojo/_base/declare",
    "dojo/_base/array",
    "dojo/_base/lang",
    "dojo/query",
    "dojo/dom-construct",
    "dojo/dom-class",
    "dijit/_Container",
    "dijit/form/RadioButton",
    "dijit/_WidgetBase",
    "epi-saas-base/form/RadioButtonList",
    "dojo/i18n!./nls/WeightSelector"
], function (declare, array, lang, query, domConstruct, domClass, _Container, RadioButton, _WidgetBase, RadioButtonList, i18n) {

    // module:
    //      epi-find/widget/WeightSelector

    return declare([_WidgetBase, _Container], {
        // summary:
        //              Weight selector for picking a weight.
        // description:
        //              The selector is configurable and can take a custom array of values and/or labels.
        //              The first item is the "none-selected" value and label that is returned if no selection was made
        //              Default value for the weight selector is 0 (None selected)
        // example:
        //              In the example below, -2 is the none-selected value, and "None" is the none-selected label
        //              Watches on the selector's value and label writes the user selections to the console
        //
        // |            <script>
        // |            require(["dojo/dom", "epi-find/widget/WeightSelector", "dojo/domReady!"],
        // |                function(dom, WeightSelector) {
        // |                    var weightContainer = dom.byId("widgetNode"),
        // |                        selector = new WeightSelector({
        // |                            value: 1,
        // |                            values: [{value: -2, label: "None"},
        // |                                     {value: -1, label: "Cold"},
        // |                                     {value: 0, label: "Mild"},
        // |                                     {value: 1, label:"Hot"}]
        // |                        });
        // |                    selector.placeAt(weightContainer);
        // |                    selector.startup();
        // |                    selector.watch("value", function (attr, oldVal, newVal) {
        // |                        console.log("Changed value from " + oldVal + " to " + newVal);
        // |                    });
        // |                    selector.watch("label", function (attr, oldVal, newVal) {
        // |                        console.log("Changed level from " + oldVal + " to " + newVal);
        // |                    });
        // |                });
        // |            </script>
        // |            <div id="widgetNode"></div>

        // value: Number
        //      Value of selected weight
        value: 0,

        _setValueAttr: function (/*Number*/ value) {
            // summary:
            //          Setter for the value attribute
            // value: Number
            //          The new value to assign to the weight selector

            //set new value and label
            this._set("value", value);
            this.set("label", this._getLabelKeyForValue(value));

            if(value == this.values[0].value){ //non-selected value
                //set checked state of the radio buttons to false
                array.forEach(this._radioButtons, function(radioButton){
                    radioButton.set("checked",false);
                    query(".is-filled", this.domNode).forEach(function(node){
                        domClass.remove(node, "is-filled");
                    });
                }, this);
            }
            else{
                //find the radio button for the specified value and set its checked state
                var radioButton = array.filter(this._radioButtons, function (item) {
                    return item.value == value;
                });
                if (radioButton.length) {
                    radioButton[0].set("checked", true);
                }
            }
        },

        // label: String
        //      Description of checked weight. Updated when value changes
        label: "",

        // Setter for the label attribute
        // Maps the attribute to _labelDomNode
        _setLabelAttr: {node: "_labelDomNode", type: "innerHTML"},

        // disabled: Boolean
        //      Disable the weight selector by setting disabled to true
        disabled: false,

        _setDisabledAttr: function(/*Boolean*/ disabled){
            //summary:
            //          Setter for the disabled attribute
            // disabled: Boolean
            //          true if weight selector should be disabled, false if enabled

            this._set("disabled", disabled);
            this._toggleState(disabled);
        },

        // values: Object[] of {value: Number, label: String}
        //      Values to use for radio buttons
        //      First value is the none-selected value
        values: null,

        // baseClass: String
        //      Css class to use for the weight selector
        baseClass: "epi-weight-selector",

        _toggleState: function(/*Boolean*/ disabled){
            // summary:
            //          toggles the radio buttons' disabled property based on disabled
            // disabled: Boolean
            //          true if radio buttons should be disabled, false if enabled

            array.forEach(this._radioButtons, function(item) {
                item.set("disabled", disabled);
            });
        },

        // _labelDomNode: DomNode
        //      Reference to the label domNode for displaying label text
        _labelDomNode: null,

        // _radioButtons: dijit/form/RadioButton[]
        //      Reference to all radio buttons for managing checked state upon value changes
        _radioButtons: null,

        constructor: function () {
            // summary:
            //          Constructs an instance of WeightSelector widget
            // options: Object[] of { value: Number, label: String }

            //default values and labels
            //keep reference in _defaultValues for postMixInProperties
            this.values = this._defaultValues = [
                { value: 0, label: i18n.none },
                { value: 1, label: i18n.veryLow },
                { value: 2, label: i18n.low },
                { value: 3, label: i18n.medium },
                { value: 4, label: i18n.high },
                { value: 5, label: i18n.veryHigh }
            ];

            this._radioButtons = [];
        },

        postMixInProperties: function(){
            // Merge value objects with default value objects
            array.forEach(this.values, function (value, idx) {
                this.values[idx] = lang.mixin({}, this._defaultValues[idx] || {}, value instanceof Object ? value : { value: value });
            }, this);
        },

        _getLabelKeyForValue: function (/*Number*/ value) {
            // summary:
            //              Get the label for the specified value
            // description:
            //              If fewer labels than values specified, and no matching label found, returns value
            //              If value does not match any in values array, return localized message "value out of range"
            // value: Number
            //              Value for which to return a label
            // returns: String

            var matches = array.filter(this.values, function(item){
                return item.value === value;
            });
            return matches.length ? matches[0].label || value : i18n.outOfRange;
        },

        _createRadioButtons: function() {
            // summary:
            //          Create and return an array of radio buttons from the values array,
            //              except for the first item (non-selected value)
            // returns: dijit/form/RadioButton[]

            var self = this,

                //create a radio button with the specified value
                createRadioBtn = function(decValue) {
                    var radBtn = new RadioButton({
                        value: decValue,
                        name: "Weight " + self.id //widget's id, thus same for all radio buttons (group)
                    });
                    return radBtn;
                },

                //generate configurable number of radio buttons, except first item (exclusion value)
                radioButtons = array.map(this.values.slice(1), function(item) {
                    return createRadioBtn(item.value);
                });
            return radioButtons;
        },

        buildRendering: function() {
            // summary:
            //          Build and render the weight selector widget

            this.inherited(arguments);
            var self = this,
                //inherit from RadioButtonList and override createListItem to manage is-filled css class
                SelectorList = declare([RadioButtonList], {
                    createListItem: function(radioButton, options) {
                        var listItem = this.inherited(arguments),
                            list = this,
                            checkedBtnIndex = this.getIndexOfChild(listItem),
                            setDomClasses = function(){
                                var children = list.getChildren();
                                //set and remove the is-filled css class depending on before or after checked radiobutton
                                array.forEach(children, function(sibling, siblingIndex){
                                    if (siblingIndex <= checkedBtnIndex) {
                                        domClass.add(sibling.domNode, "is-filled");
                                    } else {
                                        domClass.remove(sibling.domNode, "is-filled");
                                    }
                                });
                            };

                        self.own(radioButton.watch("checked", function(name, oldValue, newValue) {
                            if (newValue) {
                                //set value of widget to selected radio button
                                self.set("value", radioButton.value);
                                setDomClasses();
                            }
                        }));
                    }
                }),
                weightList = new SelectorList();

            // create radio buttons and add to container widget weightList
            this.own(weightList);
            this.addChild(weightList);
            this._radioButtons = this._createRadioButtons();
            array.forEach(this._radioButtons, function (item, radioButtonIndex) {
                self.own(item);
                weightList.createListItem(item, { blockClass: "epi-weight-selector__block", inputStyleType: radioButtonIndex+1 });
            });

            //Add label to the dom
            this._labelDomNode = domConstruct.toDom("<span id=label" + this.id + "></span>");
            domConstruct.place(this._labelDomNode, this.domNode, "last");
        }
    });
});